@@ -80,12 +80,23 @@ class File(object):
                 self._reader.SetInputSRS(self.in_srs)
             if self.out_srs:
                 self._reader.SetOutputSRS(self.out_srs)
+            ## Wire up API for extra dimensions
+            if self._reader.extra_dimensions != []:
+                for dimension in self._reader.extra_dimensions:
+                    dimname = dimension.name.replace("\x00", "").replace(" ", "_").lower()
+                    self.addProperty(dimname)
+
 
         if self._mode == 'rw':
             if self._header == None:
                 self._writer = base.Writer(self.filename,mode = self._mode)
                 self._reader = self._writer
                 self._header = self._reader.get_header()
+                ## Wire up API for any extra Dimensions
+                if self._writer.extra_dimensions != []:
+                    for dimension in self._writer.extra_dimensions:
+                        dimname = dimension.name.replace("\x00", "").replace(" ", "_").lower()
+                        self.addProperty(dimname) 
             else:
                 raise util.LaspyException("Headers must currently be stored in the file, you provided: " + str(self._header))
     
@@ -104,10 +115,25 @@ class File(object):
                     self._evlrs.extend(evlrs)
                 else:
                     self._evlrs = evlrs
+
             self._writer = base.Writer(self.filename, mode = "w",
                                       header = self._header, 
                                       vlrs = self._vlrs, evlrs = self._evlrs)
             self._reader = self._writer
+            ## Wire up API for any extra Dimensions
+            if self._writer.extra_dimensions != []:
+                for dimension in self._writer.extra_dimensions:
+                    dimname = dimension.name.replace("\x00", "").replace(" ", "_").lower()
+                    self.addProperty(dimname) 
+
+
+
+
+
+
+
+
+
         if self._mode == 'w+':
             raise NotImplementedError
 
@@ -124,6 +150,14 @@ class File(object):
             self._writer = None
             self._header = None
 
+    def addProperty(self, name):
+        def fget(self):
+            return(self._reader.get_dimension(name))
+        def fset(self, value):
+            self.assertWriteMode()
+            self._reader.set_dimension(name, value)
+        setattr(self.__class__, name, property(fget, fset, None, None))
+
     def assertWriteMode(self):
         if self._mode == "r":
             raise util.LaspyException("File is not opened in a write mode.")         
@@ -593,10 +627,11 @@ class File(object):
             else:
                 self.close()
                 self.open()
+        else:
+            print("Iteration only supported in read mode, try using FileObject.points")
+            yield(None)
 
 
-    ### END OF GB REVISIONS ###
-
     def __getitem__(self, index):
         '''Index and slicing support
 
