@@ -1,7 +1,7 @@
 import datetime
 from uuid import UUID
 import util
-from struct import pack
+from struct import unpack, pack, Struct
 import copy
 import numpy as np
 def leap_year(year):
@@ -18,6 +18,150 @@ def leap_year(year):
 class LaspyHeaderException(Exception):
     pass
 
+
+class ExtraBytesStruct(object):
+    '''This class provides a frontend for the Extra Bytes Struct as defined
+    in the LAS 1.4 specification. All Arguments are assumed to be default values
+    unless specified - the two really key ones are name and data_type. This structure
+    lives in the VLR_body property of a laspy.header.VLR instance, and multiple 
+    ExtraByteStruct records can be present together. Each ExtraBytesStruct instance
+    describes an additional dimension present at the end of each point record. ''' 
+    def __init__(self, data_type = 0, options = 0, name = "\x00"*32, 
+                 unused = [0]*4, no_data = [0.0]*3, min = [0.0]*3, 
+                 max = [0.0]*3, scale = [0.0]*3, offset = [0.0]*3, 
+                 description = "\x00"*32):
+        self.fmt = util.Format("extra_bytes_struct")
+        self.packer = Struct(self.fmt.pt_fmt_long)
+        self.writeable = True
+        self.vlr_parent = False
+        self.names = [x.name for x in self.fmt.specs]
+
+        self.data = "\x00"*192 
+        self.set_property("data_type" , data_type)
+        self.set_property("options" , options)
+        self.set_property("name" , name + "\x00"*(32-len(description)))
+        self.set_property("unused" , unused )
+        self.set_property("no_data" , no_data)
+        self.set_property("min" , min)
+        self.set_property("max" , max)
+        self.set_property("scale" , scale )
+        self.set_property("offset" , offset)
+        self.set_property("description" , description + "\x00"*(32-len(description)))
+
+    def build_from_vlr(self, vlr_parent, body_offset):
+        self.writeable = False
+        self.data = vlr_parent.VLR_body[192*body_offset:192*(body_offset + 1)]
+        self.vlr_parent = vlr_parent
+        self.body_offset = body_offset
+    
+    def assertWriteable(self):
+        if self.writeable or self.vlr_parent.reader == False:
+            return
+        raise util.LaspyException("""To modify VLRs and EVLRs, you must create new
+                            variable length records, and then replace them via 
+                            file.header.vlrs or file.header.evlrs. To do otherwise 
+                            would cause data concurrency issues.""")
+
+    def get_property_idx(self, name):
+        idx = 0
+        for i in self.names:
+            if name == i:
+                return(idx)
+            idx += 1
+        raise util.LaspyException("Property: " + str(name) + " not found. ")
+
+    def get_property(self, name):
+        fmt = self.fmt.specs[self.get_property_idx(name)]
+        unpacked = unpack(fmt.full_fmt, 
+                   self.data[fmt.offs:(fmt.offs + fmt.length*fmt.num)])
+        if len(unpacked) == 1:
+            return(unpacked[0])
+        return(unpacked)
+    
+    def to_byte_string(self):
+        '''Return the struct as a string of raw bytes. This is useful for providing
+        the struct to a VLR instance via VLR_body.'''
+        return(self.data)
+
+    def set_property(self, name, value):
+        self.assertWriteable()
+        fmt = self.fmt.specs[self.get_property_idx(name)]
+        if isinstance(value, int) or isinstance(value, str):
+            packed = pack(fmt.full_fmt, value)
+            self.data = self.data[0:fmt.offs] + packed + self.data[fmt.offs + len(packed):len(self.data)]  
+        else: 
+            packed = pack(fmt.full_fmt, *value)
+            self.data = self.data[0:fmt.offs] + packed + self.data[fmt.offs + len(packed):len(self.data)]  
+
+        if self.vlr_parent != False:
+            idx_start = 192*self.body_offset
+            idx_stop = 192*(self.body_offset + 1) 
+            d1 = self.vlr_parent.VLR_body[0:idx_start]
+            d2 = self.vlr_parent.VLR_body[idx_stop:len(self.vlr_parent.VLR_body)]
+            self.vlr_parent.VLR_body = (d1 + self.data + d2)
+        return
+
+    def get_reserved(self):
+        return(self.get_property("reserved"))
+    def set_reserved(self, value):
+        self.set_property("reserved", value)
+    reserved = property(get_reserved, set_reserved, None, None)
+
+    def get_data_type(self):
+        return(self.get_property("data_type"))
+    def set_data_type(self, value):
+        self.set_property("data_type", value)
+    data_type = property(get_data_type, set_data_type, None, None)
+
+    def get_options(self):
+        return(self.get_property("options"))
+    def set_options(self, value):
+        self.set_property("options", value)
+    options = property(get_options, set_options, None, None)
+
+    def get_name(self):
+        return(self.get_property("name"))
+    def set_name(self, value):
+        self.set_property("name", value)
+    name = property(get_name, set_name, None, None)
+
+    def get_no_data(self):
+        return(self.get_property("no_data"))
+    def set_no_data(self, value):
+        self.set_property("no_data", value)
+    no_data = property(get_no_data, set_no_data, None, None)
+
+    def get_min(self):
+        return(self.get_property("min"))
+    def set_min(self, value):
+        self.set_property("min", value)
+    min = property(get_min, set_min, None, None)
+
+    def get_max(self):
+        return(self.get_property("max"))
+    def set_max(self, value):
+        self.set_property("max", value)
+    max = property(get_max, set_max, None, None)
+
+    def get_scale(self): 
+        return(self.get_property("scale"))
+    def set_scale(self, value):
+        self.set_property("scale", value)
+    scale = property(get_scale, set_scale, None, None)
+
+    def get_offset(self):
+        return(self.get_property("offset"))
+    def set_offset(self, value):
+        self.set_property("offset", value)
+    offset = property(get_offset, set_offset, None, None)
+
+    def get_description(self):
+        return(self.get_property("description"))
+    def set_description(self):
+        self.set_property(self, "description")
+    description = property(get_description, set_description, None, None)
+
+        
 class EVLR():
     ''' An extended VLR as defined in LAS specification 1.4'''
     def __init__(self, user_id, record_id, VLR_body, **kwargs):
@@ -40,6 +184,23 @@ class EVLR():
             self.reserved = 0 
         self.fmt = util.Format("EVLR")
         self.isEVLR = True
+        self.type = 0
+        if self.user_id == "LASF_Spec" and self.record_id == 4:
+            self.setup_extra_bytes_spec(self.VLR_body)
+
+    def setup_extra_bytes_spec(self, VLR_body):
+        self.type = 1
+        self.extra_dimensions = []
+        if self.rec_len_after_header % 192 != 0:
+            raise util.LaspyException("""Invalid record length for extra bytes
+                                     specification, must be multiple of 192.""")
+        else:
+            recs = self.rec_len_after_header / 192
+            for i in xrange(recs):
+                new_rec = ExtraBytesStruct()
+                new_rec.build_from_vlr(self, i)
+            self.add_extra_dim(new_rec)
+
 
     def build_from_reader(self, reader):
         '''Build an evlr from a reader capable of reading in the data.'''
@@ -49,6 +210,9 @@ class EVLR():
         self.rec_len_after_header = reader.read_words("rec_len_after_header", "evlr")
         self.description = "".join(reader.read_words("description", "evlr"))
         self.VLR_body = reader.read(self.rec_len_after_header)
+        if self.user_id == "LASF_Spec" and self.record_id == 4:
+            self.setup_extra_bytes_spec(self.VLR_body)
+
         ### LOGICAL CONTENT ###
         self.isEVLR = True
         self.fmt = reader.evlr_formats
@@ -91,6 +255,8 @@ class VLR():
         self.user_id = str(user_id) + "\x00"*(16-len(str(user_id)))
         self.record_id = record_id
         self.VLR_body = VLR_body
+        self.type = 0
+        self.reader = False
         try:
             self.rec_len_after_header = len(self.VLR_body)
         except(TypeError):
@@ -106,19 +272,43 @@ class VLR():
         self.reserved = 0
         self.isVLR = True
         self.fmt = util.Format("VLR")
+        if self.user_id == "LASF_Spec" and self.record_id == 4:
+            self.setup_extra_bytes_spec(self.VLR_body)
 
     def build_from_reader(self, reader):
         '''Build a vlr from a reader capable of reading in the data.'''
+        self.reader = reader
         self.reserved = reader.read_words("reserved")
         self.user_id = "".join(reader.read_words("user_id"))
         self.record_id = reader.read_words("record_id")
         self.rec_len_after_header = reader.read_words("rec_len_after_header")
         self.description = "".join(reader.read_words("description"))
         self.VLR_body = reader.read(self.rec_len_after_header)
+        if "LASF_Spec" in self.user_id and self.record_id == 4:
+            self.setup_extra_bytes_spec(self.VLR_body)
         ### LOGICAL CONTENT ###
         self.isVLR = True
         self.fmt = reader.vlr_formats
 
+    def setup_extra_bytes_spec(self, VLR_body):
+        self.type = 1
+        self.extra_dimensions = []
+        if self.rec_len_after_header % 192 != 0:
+            raise util.LaspyException("""Invalid record length for extra bytes
+                                     specification, must be multiple of 192.""")
+        else:
+            recs = self.rec_len_after_header / 192
+            for i in xrange(recs):
+                new_rec = ExtraBytesStruct()
+                new_rec.build_from_vlr(self, i) 
+                self.add_extra_dim(new_rec)
+        
+    def add_extra_dim(self, new_rec):
+        new_name = new_rec.name.replace("\x00", "").replace(" ", "_").lower()
+        self.__dict__[new_name] = new_rec
+        self.extra_dimensions.append(new_rec)
+
+
     def __len__(self):
         '''Return the size of the vlr object in bytes'''
         return self.rec_len_after_header + 54
@@ -177,7 +367,8 @@ class Header(object):
         for item in self._format.specs:
             if not item.name in new_format.lookup.keys(): 
                 self.__dict__[item.name] = None
-        
+        if str(file_version) == "1.4":
+            self.point_return_count.extend([0]*10)
         self._format = new_format
 
     def get_format(self):
@@ -704,7 +895,8 @@ class HeaderManager(object):
         #for i in rawdata:
         #    histDict[i] += 1        
         #raw_hist = histDict.values()
-        if self.data_format_id in (range(6)):
+        #if self.data_format_id in (range(6)):
+        if self.version != "1.4":
             raw_hist = np.histogram(rawdata, bins = range(1,7))
             
         else:
